import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.text.*;

public class CoronalMenu extends JPanel implements ActionListener {
    /*
     * Note: To change the default value, just change the constant below.
     */

    // Output parameter
    private static final boolean VR = true;
    private static final boolean VT = true;
    private static final boolean VP = true;
    private static final boolean BR = true;
    private static final boolean BT = true;
    private static final boolean BP = true;
    private static final boolean RHO = true;
    private static final boolean T = true;
    private static final boolean P = true;
    private static final boolean JR = false;
    private static final boolean JT = false;
    private static final boolean JP = false;
    private static final boolean AR = false;
    private static final boolean AT = false;
    private static final boolean AP = false;

    private static final int TMAX = 120;         // Maximum time
    private static final int OUTRAD = 30;        // Outer radial boundary
    private static final int OUTINT = TMAX;      // Output interval
    private static final float DTMIN = 0.005f;   // dt Min
    private static final float DTMAX = 0.05f;    // dt Max
    private static final float LUND = 1.0e3f;    // Lundquist number
    private static final float VISC = 0.01f;     // Viscosity
    private static final float DENSITY = 2.0f;   // Density
    private static final float TEMP = 1.8e6f;    // Temperature


    private JFrame frame;
    private JCheckBox vr, vt, vp, br, bt, bp, rho, t, p, jr, jt, jp,
                      ar, at, ap;
    private JTextField tmax, outRad, outInt, dtMin, dtMax, lund, visc, density,
                       temp;
    private JButton saveButton;

    public CoronalMenu () {
        super (new GridBagLayout ());

        frame = new JFrame ("Coronal Options");
        setOpaque (true);
        setBorder (BorderFactory.createRaisedBevelBorder ());
        frame.setContentPane (this);
        frame.setResizable (false);

        GridBagConstraints c = new GridBagConstraints ();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.insets = new Insets (5, 5, 5, 5);

        // Output parameters
        JLabel outputParamLabel = new JLabel ("Output Parameters:");
        add (outputParamLabel, c);

        // Output parameters check box container
        JPanel outputParamPanel = new JPanel (new GridLayout (3, 5, 10, 10));
        vr = new JCheckBox ("Vr", VR);
        vt = new JCheckBox ("Vt", VT);
        vp = new JCheckBox ("Vp", VP);
        br = new JCheckBox ("Br", BR);
        bt = new JCheckBox ("Bt", BT);
        bp = new JCheckBox ("Bp", BP);
        rho = new JCheckBox ("Rho", RHO);
        t = new JCheckBox ("T", T);
        p = new JCheckBox ("P", P);
        jr = new JCheckBox ("Jr", JR);
        jt = new JCheckBox ("Jt", JT);
        jp = new JCheckBox ("Jp", JP);
        ar = new JCheckBox ("ar", AR);
        at = new JCheckBox ("at", AT);
        ap = new JCheckBox ("ap", AP);
        outputParamPanel.add (vr);
        outputParamPanel.add (br);
        outputParamPanel.add (rho);
        outputParamPanel.add (jr);
        outputParamPanel.add (ar);
        outputParamPanel.add (vt);
        outputParamPanel.add (bt);
        outputParamPanel.add (t);
        outputParamPanel.add (jt);
        outputParamPanel.add (at);
        outputParamPanel.add (vp);
        outputParamPanel.add (bp);
        outputParamPanel.add (p);
        outputParamPanel.add (jp);
        outputParamPanel.add (ap);

        add (outputParamPanel, c);

        // Input panel container
        JPanel inputPanel = new JPanel (new GridLayout (9, 2, 10, 10));		

        tmax = new JTextField (String.valueOf (TMAX));
        outRad = new JTextField (String.valueOf (OUTRAD));
        outInt = new JTextField (String.valueOf (OUTINT));
        dtMax = new JTextField (String.valueOf (DTMAX));
        dtMin = new JTextField (String.valueOf (DTMIN));
        lund = new JTextField (String.valueOf (LUND));
        visc = new JTextField (String.valueOf (VISC));
        density = new JTextField (String.valueOf (DENSITY));
        temp = new JTextField (String.valueOf (TEMP));
        inputPanel.add (new JLabel ("Maximum Time:"));
        inputPanel.add (tmax);
        inputPanel.add (new JLabel ("Outer Radial Boundary:"));
        inputPanel.add (outRad);
        inputPanel.add (new JLabel ("Output Interval:"));
        inputPanel.add (outInt);
        inputPanel.add (new JLabel ("dt Max:"));
        inputPanel.add (dtMax);
        inputPanel.add (new JLabel ("dt Min:"));
        inputPanel.add (dtMin);
        inputPanel.add (new JLabel ("Lundquist Number:"));
        inputPanel.add (lund);
        inputPanel.add (new JLabel ("Viscosity"));
        inputPanel.add (visc);
        inputPanel.add (new JLabel ("Density"));
        inputPanel.add (density);
        inputPanel.add (new JLabel ("Temperature"));
        inputPanel.add (temp);

        add (inputPanel, c);

        // Save all of the coronal input parameters
        saveButton = new JButton ("Save Input File Parameters");
        saveButton.addActionListener (this);
        saveButton.setActionCommand ("save");
        add (saveButton, c);

        frame.pack ();
        frame.setVisible (false);
    }

    public String getOutputParams () {
        int counter = 0;
        String outputParams = "";

        if (vr.isSelected ()) {
            outputParams += (counter == 0) ? "'vr'" : ",'vr'";
            counter++;
        }
        if (vt.isSelected ()) {
            outputParams += (counter == 0) ? "'vt'" : ",'vt'";
            counter++;
        }
        if (vp.isSelected ()) {
            outputParams += (counter == 0) ? "'vp'" : ",'vp'";
            counter++;
        }
        if (br.isSelected ()) {
            outputParams += (counter == 0) ? "'br'" : ",'br'";
            counter++;
        }
        if (bt.isSelected ()) {
            outputParams += (counter == 0) ? "'bt'" : ",'bt'";
            counter++;
        }
        if (bp.isSelected ()) {
            outputParams += (counter == 0) ? "'bp'" : ",'bp'";
            counter++;
        }
        if (rho.isSelected ()) {
            outputParams += (counter == 0) ? "'rho'" : ",'rho'";
            counter++;
        }
        if (t.isSelected ()) {
            outputParams += (counter == 0) ? "'t'" : ",'t'";
            counter++;
        }
        if (p.isSelected ()) {
            outputParams += (counter == 0) ? "'p'" : ",'p'";
            counter++;
        }
        if (jr.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'jr'" : ",'jr'";
            counter++;
        }
        if (jt.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'jt'" : ",'jt'";
            counter++;
        }
        if (jp.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'jp'" : ",'jp'";
            counter++;
        }
        if (ar.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'ar'" : ",'ar'";
            counter++;
        }
        if (at.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'at'" : ",'at'";
            counter++;
        }
        if (ap.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'ap'" : ",'ap'";
            counter++;
        }
        return outputParams;
    }

    public String getMaxTime () {
        return tmax.getText ();
    }

    public String getOuterRadial () {
        return String.valueOf (Integer.parseInt (outRad.getText ()) - 1);
    }

    public String getOutputInt () {
        return outInt.getText ();
    }

    public String getDtMax () {
        return dtMax.getText ();
    }

    public String getDtMin () {
        return dtMin.getText ();
    }

    public String getLundNum () {
        return lund.getText ();
    }

    public String getViscosity () {
        return visc.getText ();
    }

    public String getDensity () {
        return density.getText ();
    }

    public String getTemperature () {
        return temp.getText ();
    }

    public void showMenu () {
        frame.setVisible (true);
    }

    public void hideMenu () {
        frame.setVisible (false);
    }

    public void reset () {
        vr.setSelected (VR);
        vt.setSelected (VT);
        vp.setSelected (VP);
        br.setSelected (BR);
        bt.setSelected (BT);
        bp.setSelected (BP);
        rho.setSelected (RHO);
        t.setSelected (T);
        p.setSelected (P);
        jr.setSelected (JR);
        jt.setSelected (JT);
        jp.setSelected (JP);
        ar.setSelected (AR);
        at.setSelected (AT);
        ap.setSelected (AP);

        tmax.setText (String.valueOf (TMAX));
        outRad.setText (String.valueOf (OUTRAD));
        outInt.setText (String.valueOf (OUTINT));
        dtMax.setText (String.valueOf (DTMAX));
        dtMin.setText (String.valueOf (DTMIN));
        lund.setText (String.valueOf (LUND));
        visc.setText (String.valueOf (VISC));
        density.setText (String.valueOf (DENSITY));
        temp.setText (String.valueOf (TEMP));

        // Resave the input parameters
        actionPerformed (new ActionEvent (saveButton,
                         ActionEvent.ACTION_PERFORMED, "save"));
    }

    public void saveInput () {
        int tmaxParse;
        try {
            tmaxParse = Integer.parseInt (tmax.getText ());
        } catch (NumberFormatException exc) {
            tmaxParse = TMAX;
            tmax.setText (Integer.toString (tmaxParse));
        }
        if (tmaxParse < 1) {
            tmaxParse = 1;
            tmax.setText (Integer.toString (tmaxParse));
        }

        int outRadParse;
        try {
            outRadParse = Integer.parseInt (outRad.getText ());
        } catch (NumberFormatException exc) {
            outRadParse = OUTRAD;
            outRad.setText (Integer.toString (outRadParse));
        }
        if (outRadParse > 30) {
            outRadParse = 30;
            outRad.setText (Integer.toString (outRadParse));
        }
        if (outRadParse < 1) {
            outRadParse = 1;
            outRad.setText (Integer.toString (outRadParse));
        }

        int outIntParse;
        try {
            outIntParse = Integer.parseInt (outInt.getText ());
        } catch (NumberFormatException exc) {
            outIntParse = OUTINT;
            outInt.setText (Integer.toString (outIntParse));
        }
        if (outIntParse > tmaxParse) { 
            outIntParse = tmaxParse;
            outInt.setText (Integer.toString (outIntParse));
        }
        if (outIntParse < 1) {
            outIntParse = 1;
            outInt.setText (Integer.toString (outIntParse));
        }

        double dtMaxParse;
        try {
            dtMaxParse = Double.parseDouble (dtMax.getText ());
        } catch (NumberFormatException exc) {
            dtMaxParse = DTMAX;
            dtMax.setText (Double.toString (dtMaxParse));
        }
        if (dtMaxParse > 1.0) {
            dtMaxParse = 1.0;
            dtMax.setText (Double.toString (dtMaxParse));
        }
        if (dtMaxParse < 0.01) {
            dtMaxParse = 0.01;
            dtMax.setText (Double.toString (dtMaxParse));
        }

        double dtMinParse;
        try {
            dtMinParse = Double.parseDouble (dtMin.getText ());
        } catch (NumberFormatException exc) {
            dtMinParse = DTMIN;
            dtMin.setText (Double.toString (dtMinParse));
        }
        if (dtMinParse > dtMaxParse) {
            dtMinParse = dtMaxParse;
            dtMin.setText (Double.toString (dtMinParse));
        }
        if (dtMinParse < 0.001) {
            dtMinParse = 0.001; 
            dtMin.setText (Double.toString (dtMinParse));
        }

        double lundParse;
        try {
            lundParse = Double.parseDouble (lund.getText ());
        } catch (NumberFormatException exc) {
            lundParse = LUND;
            lund.setText (Double.toString (lundParse));
        }
        if (lundParse < 0.0) {
            lundParse = 0;
            lund.setText (Double.toString (lundParse));
        }

        double viscParse;
        try {
            viscParse = Double.parseDouble (visc.getText());
        } catch (NumberFormatException exc) {
            viscParse = VISC;
            visc.setText (Double.toString (viscParse));
        }
        if (viscParse < 0.0) {
            viscParse = 0;
            visc.setText (Double.toString (viscParse));
        }

        double densityParse;
        try {
            densityParse = Double.parseDouble (density.getText ());
        } catch (NumberFormatException exc) {
            densityParse = DENSITY;
            density.setText (Double.toString (densityParse));
        }
        if (densityParse < 0.0) {
            densityParse = 0.0;
            density.setText (Double.toString (densityParse));
        }

        double tempParse;
        try {
            tempParse = Double.parseDouble (temp.getText ());
        } catch (NumberFormatException exc) {
            tempParse = TEMP;
            temp.setText (Double.toString (tempParse));
        }
        if (tempParse < 0.0) {
            tempParse = 0.0;
            temp.setText (Double.toString (tempParse));
        }
    }

    public void actionPerformed (ActionEvent e) {
        if (e.getActionCommand ().equals ("save")) {
            saveInput ();
            hideMenu ();
        }
    }
}
